@@ -55,8 +55,8 @@ main()
     sscanf(tt,"%s %s",a,b);
     memcpy(&t2s[t2sn].a, a, sizeof(t2s[0].a));
     memcpy(&t2s[t2sn].b, b, sizeof(t2s[0].b));
-    memcpy(&s2t[t2sn].a, a, sizeof(s2t[0].a));
-    memcpy(&s2t[t2sn].b, b, sizeof(s2t[0].b));
+    memcpy(&s2t[t2sn].b, a, sizeof(s2t[0].a));
+    memcpy(&s2t[t2sn].a, b, sizeof(s2t[0].b));
     t2sn++;
 //    dbg("%s %s\n", a,b);
   }
