@@ -16,6 +16,10 @@ extern gboolean win_kbm_inited;
 static char *callback_str_buffer;
 Window focus_win;
 static int timeout_handle;
+char *output_buffer;
+int output_bufferN;
+static char *output_buffer_raw, *output_buffer_raw_bak;
+static int output_buffer_rawN;
 
 void send_fake_key_eve(KeySym key)
 {
@@ -24,24 +28,48 @@ void send_fake_key_eve(KeySym key)
   XTestFakeKeyEvent(dpy, kc, False, CurrentTime);
 }
 
-
-void send_text_call_back(char *text)
+void fake_shift()
 {
-  callback_str_buffer = realloc(callback_str_buffer, strlen(text));
-  strcpy(callback_str_buffer, text);
   send_fake_key_eve(XK_Shift_L);
 }
 
+void swap_ptr(char **a, char **b)
+{
+  char *t = *a;
+  *a = *b;
+  *b = t;
+}
+
+int force_preedit=0;
+void force_preedit_shift()
+{
+  send_fake_key_eve(XK_Shift_L);
+  force_preedit=1;
+}
+
+void send_text_call_back(char *text)
+{
+  callback_str_buffer = realloc(callback_str_buffer, strlen(text)+1);
+  strcpy(callback_str_buffer, text);
+  fake_shift();
+}
+
+void output_buffer_call_back()
+{
+  swap_ptr(&callback_str_buffer, &output_buffer);
+
+  if (output_buffer)
+    output_buffer[0] = 0;
+  output_bufferN = 0;
+
+  fake_shift();
+}
 
 ClientState *current_CS;
 static ClientState temp_CS;
 
 gboolean init_in_method(int in_no);
 
-char *output_buffer;
-int output_bufferN;
-static char *output_buffer_raw, *output_buffer_raw_bak;
-static int output_buffer_rawN;
 
 void clear_output_buffer()
 {
@@ -49,9 +77,7 @@ void clear_output_buffer()
     output_buffer[0] = 0;
   output_bufferN = 0;
 
-  char *t = output_buffer_raw;
-  output_buffer_raw = output_buffer_raw_bak;
-  output_buffer_raw_bak = t;
+  swap_ptr(&output_buffer_raw, &output_buffer_raw_bak);
 
   if (output_buffer_raw)
     output_buffer_raw[0] = 0;
@@ -246,14 +272,16 @@ void hide_in_win(ClientState *cs)
     case 10:
       hide_win_int();
       break;
+#if USE_ANTHY
+    case 12:
+      hide_win_anthy();
+      break;
+#endif
     default:
       hide_win_gtab();
   }
 
   reset_current_in_win_xy();
-#if 0
-  hide_win_status();
-#endif
 }
 
 void show_win_pho();
@@ -284,6 +312,11 @@ void show_in_win(ClientState *cs)
     case 10:
       show_win_int();
       break;
+#if USE_ANTHY
+    case 12:
+      show_win_anthy();
+      break;
+#endif
     default:
       show_win_gtab();
   }
@@ -333,6 +366,11 @@ void move_in_win(ClientState *cs, int x, int y)
     case 10:
       move_win_int(x, y);
       break;
+#if USE_ANTHY
+    case 12:
+      move_win_anthy(x, y);
+      break;
+#endif
     default:
       if (!cs->in_method)
         return;
@@ -345,7 +383,7 @@ static int xerror_handler(Display *d, XErrorEvent *eve)
   return 0;
 }
 
-static void getRootXY(Window win, int wx, int wy, int *tx, int *ty)
+void getRootXY(Window win, int wx, int wy, int *tx, int *ty)
 {
   Window ow;
   void *olderr = XSetErrorHandler((XErrorHandler)xerror_handler);
@@ -506,6 +544,7 @@ void toggle_im_enabled(u_int kev_state)
 #endif
       }
       else {
+        output_gbuf();
         reset_gtab_all();
       }
 
@@ -544,7 +583,7 @@ void get_win_pho_geom();
 
 void update_active_in_win_geom()
 {
-//  dbg("update_active_in_win_geom\n");
+  dbg("update_active_in_win_geom\n");
   switch (current_CS->in_method) {
     case 3:
       get_win_pho_geom();
@@ -556,6 +595,9 @@ void update_active_in_win_geom()
 #endif
     case 10:
       break;
+    case 12:
+      get_win_anthy_geom();
+      break;
     default:
       get_win_gtab_geom();
       break;
@@ -563,6 +605,7 @@ void update_active_in_win_geom()
 }
 
 extern GtkWidget *gwin_pho, *gwin0, *gwin_gtab;
+int anthy_visible();
 
 gboolean win_is_visible()
 {
@@ -574,10 +617,13 @@ gboolean win_is_visible()
 #if USE_TSIN
     case 6:
       return GTK_WIDGET_VISIBLE(gwin0);
-      break;
 #endif
     case 10:
       break;
+#if USE_ANTHY
+    case 12:
+      return anthy_visible();
+#endif
     default:
       if (!gwin_gtab)
         return FALSE;
@@ -656,7 +702,8 @@ gboolean init_in_method(int in_no)
 #if USE_TSIN
       flush_tsin_buffer();
 #endif
-      }
+      } else
+        output_gbuf();
 
       hide_in_win(current_CS);
     }
@@ -684,6 +731,16 @@ gboolean init_in_method(int in_no)
       current_CS->in_method = in_no;
       init_inter_code();
       break;
+#if USE_ANTHY
+    case 12:
+      if (init_win_anthy()) {
+        current_CS->in_method = in_no;
+        show_win_anthy();
+      } else {
+        return FALSE;
+      }
+      break;
+#endif
     default:
       init_gtab(in_no);
       if (!inmd[in_no].DefChars)
@@ -741,14 +798,17 @@ gboolean full_char_proc(KeySym keysym)
 
   utf8cpy(tt, s);
 
-  send_text(tt);
+  if (current_CS->in_method == 6 && current_CS->im_state == GCIN_STATE_CHINESE)
+    add_to_tsin_buf_str(tt);
+  else
+  if (gtab_phrase_on())
+    insert_gbuf_cursor1(tt);
+  else
+    send_text(tt);
+
   return 1;
 }
 
-
-
-
-
 int feedkey_pho(KeySym xkey, int kbstate);
 int feedkey_pp(KeySym xkey, int state);
 int feedkey_gtab(KeySym key, int kbstate);
@@ -765,6 +825,9 @@ gboolean timeout_raise_window()
   return FALSE;
 }
 
+extern Window xwin_pho, xwin0, xwin_gtab;
+
+
 // return TRUE if the key press is processed
 gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 {
@@ -782,13 +845,18 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     return TRUE;
   }
 
+  if (force_preedit) {
+    force_preedit=0;
+    return 1;
+  }
+
   if (keysym == XK_space) {
 #if 0
     dbg("state %x\n", kev->state);
     dbg("%x\n", Mod4Mask);
 #endif
     if (
-      ((kev_state & ControlMask) && gcin_im_toggle_keys==Control_Space) ||
+      ((kev_state & (ControlMask|Mod1Mask|ShiftMask))==ControlMask && gcin_im_toggle_keys==Control_Space) ||
       ((kev_state & Mod1Mask) && gcin_im_toggle_keys==Alt_Space) ||
       ((kev_state & ShiftMask) && gcin_im_toggle_keys==Shift_Space) ||
       ((kev_state & Mod4Mask) && gcin_im_toggle_keys==Windows_Space)
@@ -851,8 +919,11 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       return TRUE;
     }
 
+    if (!gcin_enable_ctrl_alt_switch)
+      return FALSE;
+
     int kidx = gcin_switch_keys_lookup(keysym);
-    if (kidx < 0)
+    if (kidx <= 0)
       return FALSE;
 
     current_CS->im_state = GCIN_STATE_CHINESE;
@@ -866,7 +937,6 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     return FALSE;
   }
 
-
   if (!current_CS->b_gcin_protocol) {
   if (((keysym == XK_Control_L || keysym == XK_Control_R)
                    && (kev_state & ShiftMask)) ||
@@ -897,6 +967,10 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 #endif
     case 10:
       return feedkey_intcode(keysym);
+#if USE_ANTHY
+    case 12:
+      return feedkey_anthy(keysym, kev_state);
+#endif
     default:
       return feedkey_gtab(keysym, kev_state);
   }
@@ -936,6 +1010,10 @@ gboolean ProcessKeyRelease(KeySym keysym, u_int kev_state)
   switch(current_CS->in_method) {
     case 6:
       return feedkey_pp_release(keysym, kev_state);
+#if USE_ANTHY
+    case 12:
+      return feedkey_anthy_release(keysym, kev_state);
+#endif
   }
 #endif
 
@@ -991,6 +1069,7 @@ int gcin_FocusIn(ClientState *cs)
 {
   Window win = cs->client_win;
 
+//  dbg("gcin_FocusIn\n");
   if (skip_window(win))
     return;
 
@@ -1059,7 +1138,6 @@ int xim_gcin_FocusIn(IMChangeFocusStruct *call_data)
 }
 #endif
 
-extern Window xwin_pho, xwin0, xwin_gtab;
 
 gint64 current_time();
 
@@ -1069,6 +1147,7 @@ int gcin_FocusOut(ClientState *cs)
 {
   gint64 t = current_time();
 
+//  dbg("gcin_FocusOut\n");
   if (skip_window(cs->client_win))
     return;
 
@@ -1095,6 +1174,65 @@ int gcin_FocusOut(ClientState *cs)
   return True;
 }
 
+#include "im-client/gcin-im-client-attr.h"
+
+int gcin_get_preedit(ClientState *cs, char *str, GCIN_PREEDIT_ATTR attr[], int *cursor)
+{
+//  dbg("gcin_get_preedit %x\n", current_CS);
+  if (!current_CS) {
+empty:
+//    dbg("empty\n");
+    str[0]=0;
+    *cursor=0;
+    return 0;
+  }
+
+  switch(current_CS->in_method) {
+    case 3:
+    case 10:
+      goto empty;
+    case 6:
+      return tsin_get_preedit(str, attr, cursor);
+#if USE_ANTHY
+    case 12:
+      return anthy_get_preedit(str, attr, cursor);
+#endif
+    default:
+      return gtab_get_preedit(str, attr, cursor);
+//      dbg("metho %d\n", current_CS->in_method);
+  }
+
+  return 0;
+}
+
+
+
+void gcin_reset()
+{
+  if (!current_CS)
+    return;
+//  dbg("gcin_reset\n");
+  switch(current_CS->in_method) {
+    case 3:
+      pho_reset();
+      return;
+    case 10:
+      return;
+    case 6:
+      tsin_reset();
+      return;
+#if USE_ANTHY
+    case 12:
+      gcin_anthy_reset();
+      return;
+#endif
+    default:
+      gtab_reset();
+//      dbg("metho %d\n", current_CS->in_method);
+  }
+}
+
+
 #if USE_XIM
 int xim_gcin_FocusOut(IMChangeFocusStruct *call_data)
 {
