@@ -34,6 +34,14 @@ void get_gcin_conf_fname(char *name, char fname[])
   strcat(strcat(fname,"/config/"),name);
 }
 
+void get_gcin_user_or_sys_fname(char *name, char fname[])
+{
+  if (!getenv("GCIN_TABLE_DIR"))
+    get_gcin_user_fname(name, fname);
+  else
+    get_sys_table_file_name(name, fname);
+}
+
 void get_gcin_conf_str(char *name, char **rstr, char *default_str)
 {
   char fname[MAX_GCIN_STR];
@@ -115,10 +123,6 @@ char *get_gcin_xim_name()
 {
   char *xim_name;
 
-  if ((xim_name=getenv("GCIN_XIM"))) {
-//    dbg("GCIN_XIM is set with %s\n", xim_name);
-    return xim_name;
-  }
   if ((xim_name=getenv("XMODIFIERS"))) {
     static char find[] = "@im=";
     static char sstr[32];
