@@ -8,7 +8,7 @@ int default_input_method;
 int left_right_button_tips;
 int gcin_im_toggle_keys;
 int gcin_capslock_lower, gcin_eng_phrase_enabled, gcin_init_im_enabled;
-int gcin_win_sym_click_close;
+int gcin_win_sym_click_close, gcin_edit_display;
 
 int gtab_dup_select_bell;
 int gtab_space_auto_first;
@@ -16,7 +16,6 @@ int gtab_auto_select_by_phrase;
 int gtab_press_full_auto_send;
 int gtab_pre_select;
 int gtab_disp_partial_match;
-int gtab_simple_win=1;
 int gtab_disp_key_codes;
 int gtab_disp_im_name;
 int gtab_invalid_key_in;
@@ -36,17 +35,15 @@ int phonetic_speak;
 char *phonetic_speak_sel;
 int tsin_chinese_english_toggle_key;
 int gcin_font_size_tsin_pho_in;
-int pho_simple_win=1;
 int tsin_space_opt;
 int tsin_buffer_size, tsin_tail_select_key;
 int tsin_buffer_editing_mode;
-int gcin_chars_big5_only;
 int gcin_flags_im_enabled;
 int gcin_shift_space_eng_full;
 char *tsin_phrase_line_color;
 char *tsin_cursor_color, *gcin_sel_key_color;
 int tsin_tab_phrase_end;
-int gcin_input_style, gcin_root_x, gcin_root_y, gcin_pop_up_win, gcin_pop_up_win_abs_corner;
+int gcin_input_style, gcin_root_x, gcin_root_y, gcin_pop_up_win;
 int gcin_inner_frame;
 char *gcin_font_name, *gcin_win_color_fg, *gcin_win_color_bg;
 #if TRAY_ENABLED
@@ -54,6 +51,12 @@ int gcin_status_tray;
 #endif
 
 int pho_hide_row2, pho_in_row1;
+#if USE_GCB
+int gcb_enabled, gcb_position, gcb_position_x, gcb_position_y;
+#endif
+int gcin_bell_volume;
+int gcin_sound_play_overlap, gcin_enable_ctrl_alt_switch;
+
 
 int get_gcin_conf_int(char *name, int default_value);
 
@@ -73,14 +76,7 @@ void load_setttings()
   gcin_inner_frame = get_gcin_conf_int(GCIN_INNER_FRAME, 0);
   gcin_eng_phrase_enabled = get_gcin_conf_int(GCIN_ENG_PHRASE_ENABLED, 1);
   gcin_init_im_enabled = get_gcin_conf_int(GCIN_INIT_IM_ENABLED, 0);
-#if GTK_MAJOR_VERSION >=2 && GTK_MINOR_VERSION >= 4
-#define ABS_VAL 0
-#else
-#define ABS_VAL 1
-#endif
-  gcin_pop_up_win_abs_corner = get_gcin_conf_int(GCIN_POP_UP_WIN_ABS_CORNER, ABS_VAL);
 
-  gcin_chars_big5_only = get_gcin_conf_int(GCIN_CHARS_BIG5_ONLY,0);
   gcin_flags_im_enabled = get_gcin_conf_int(GCIN_FLAGS_IM_ENABLED,0x7fffffff);
   gcin_remote_client = get_gcin_conf_int(GCIN_REMOTE_CLIENT, 0);
   gcin_shift_space_eng_full = get_gcin_conf_int(GCIN_SHIFT_SPACE_ENG_FULL, 1);
@@ -96,11 +92,10 @@ void load_setttings()
 
   gtab_dup_select_bell = get_gcin_conf_int(GTAB_DUP_SELECT_BELL, 0);
   gtab_space_auto_first = get_gcin_conf_int(GTAB_SPACE_AUTO_FIRST, GTAB_space_auto_first_none);
-  gtab_auto_select_by_phrase = get_gcin_conf_int(GTAB_AUTO_SELECT_BY_PHRASE, 1);
+  gtab_auto_select_by_phrase = get_gcin_conf_int(GTAB_AUTO_SELECT_BY_PHRASE, 0);
   gtab_pre_select = get_gcin_conf_int(GTAB_PRE_SELECT, 1);
   gtab_press_full_auto_send = get_gcin_conf_int(GTAB_PRESS_FULL_AUTO_SEND, 1);
   gtab_disp_partial_match = get_gcin_conf_int(GTAB_DISP_PARTIAL_MATCH, 1);
-  gtab_simple_win = get_gcin_conf_int(GTAB_SIMPLE_WIN, 1);
   gtab_disp_key_codes = get_gcin_conf_int(GTAB_DISP_KEY_CODES, 1);
   gtab_disp_im_name = get_gcin_conf_int(GTAB_DISP_IM_NAME, 1);
   gtab_invalid_key_in = get_gcin_conf_int(GTAB_INVALID_KEY_IN, 1);
@@ -117,7 +112,6 @@ void load_setttings()
                                     TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock);
   tsin_tone_char_input = get_gcin_conf_int(TSIN_TONE_CHAR_INPUT, 0);
 
-  pho_simple_win = get_gcin_conf_int(PHO_SIMPLE_WIN, 1);
   tsin_space_opt = get_gcin_conf_int(TSIN_SPACE_OPT, TSIN_SPACE_OPT_SELECT_CHAR);
   tsin_buffer_size = get_gcin_conf_int(TSIN_BUFFER_SIZE, 40);
   tsin_tab_phrase_end = get_gcin_conf_int(TSIN_TAB_PHRASE_END, 0);
@@ -139,4 +133,22 @@ void load_setttings()
   get_gcin_conf_str(GCIN_WIN_COLOR_FG, &gcin_win_color_fg, "white");
   get_gcin_conf_str(GCIN_WIN_COLOR_BG, &gcin_win_color_bg, "#005BFF");
   gcin_win_color_use = get_gcin_conf_int(GCIN_WIN_COLOR_USE, 0);
+
+
+#if USE_GCB
+  gcb_enabled = get_gcin_conf_int(GCB_ENABLED, 0);
+  gcb_position = get_gcin_conf_int(GCB_POSITION, 4);
+  gcb_position_x = get_gcin_conf_int(GCB_POSITION_X, 0);
+  gcb_position_y = get_gcin_conf_int(GCB_POSITION_Y, 0);
+#endif
+  gcin_bell_volume = get_gcin_conf_int(GCIN_BELL_VOLUME, -97);
+  gcin_sound_play_overlap = get_gcin_conf_int(GCIN_SOUND_PLAY_OVERLAP, 0);
+  gcin_enable_ctrl_alt_switch = get_gcin_conf_int(GCIN_ENABLE_CTRL_ALT_SWITCH, 1);
+#if 1
+  gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_BOTH);
+#elif 1
+  gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_ON_THE_SPOT);
+#else
+  gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_OVER_THE_SPOT);
+#endif
 }
