@@ -207,7 +207,7 @@ int main(int argc, char **argv)
   bzero(itout64,sizeof(itout64));
 
   cmd_arg(&cmd, &arg);
-  if (sequ(cmd, "%gen_inp") || sequ(cmd, "%encoding")) {
+  if (sequ(cmd, "%gen_inp")) {
     dbg("skip gen_inp\n");
     cmd_arg(&cmd, &arg);
   }
@@ -257,6 +257,10 @@ int main(int argc, char **argv)
     cmd_arg(&cmd, &arg);
   }
 
+  if (sequ(cmd,"%phase_auto_skip_endkey")) {
+    th.flag |= FLAG_PHRASE_AUTO_SKIP_ENDKEY;
+    cmd_arg(&cmd, &arg);
+  }
 
   if (!sequ(cmd,"%keyname") || !sequ(arg,"begin")) {
     p_err("%d:  %%keyname begin   expected, instead of %s %s", lineno, cmd, arg);
@@ -523,6 +527,7 @@ int main(int argc, char **argv)
 
   fwrite(idx1, sizeof(gtab_idx1_t), KeyNum+1, fw);
 
+
   if (key64) {
 #if NEED_SWAP
     for(i=0; i < chno; i++) {
