@@ -1,5 +1,7 @@
 #include "gcin.h"
 #include "pho.h"
+#include "config.h"
+#include <libintl.h>
 
 GtkWidget *hbox_buttons;
 char current_str[MAX_PHRASE_LEN*CH_SZ+1];
@@ -126,6 +128,13 @@ gboolean cb_button_fetch()
 
 int main(int argc, char **argv)
 {
+
+#if GCIN_i18n_message
+  gtk_set_locale();
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+
   gtk_init (&argc, &argv);
 
   char kbm_str[32];
@@ -169,12 +178,12 @@ int main(int argc, char **argv)
   hbox_buttons = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox_top), hbox_buttons, FALSE, FALSE, 0);
 
-  GtkWidget *button_fetch = gtk_button_new_with_label("自剪貼區更新");
+  GtkWidget *button_fetch = gtk_button_new_with_label(_("自剪貼區更新"));
   gtk_box_pack_start (GTK_BOX (hbox_buttons), button_fetch, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button_fetch), "clicked",
      G_CALLBACK (cb_button_fetch), NULL);
 
-  GtkWidget *button_exit = gtk_button_new_with_label("離開");
+  GtkWidget *button_exit = gtk_button_new_with_label(_("離開"));
   gtk_box_pack_start (GTK_BOX (hbox_buttons), button_exit, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button_exit), "clicked",
      G_CALLBACK (do_exit), NULL);
