@@ -6,6 +6,8 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include "gcin.h"
+#include "gtab.h"
+
 
 struct keystruc {
   char *kname;
@@ -149,23 +151,42 @@ gboolean feed_phrase(KeySym ksym, int state)
     trN = tranN;
   }
 
+  extern int gbufN;
+  char tt[2], *str;
 
   for(i=0; i < trN; i++) {
     if (tr[i].ksym!= ksym)
       continue;
-    char *str;
 
     str = ((state & LockMask) && tr[i].str_caps) ? tr[i].str_caps : tr[i].str;
 
     if (str) {
-      if (current_CS->in_method == 6 && current_CS->im_state == GCIN_STATE_CHINESE)
+send_it:
+      if (current_CS->in_method == 6 && current_CS->im_state == GCIN_STATE_CHINESE) {
         add_to_tsin_buf_str(str);
+        if (tsin_cursor_end())
+          flush_tsin_buffer();
+      }
       else
+      if (gtab_phrase_on()) {
+        insert_gbuf_cursor1(str);
+        if (gtab_cursor_end())
+          output_gbuf();
+      } else
         send_text(str);
-    }
 
-    return str!=NULL;
+      return TRUE;
+    }
   }
 
+#if 1
+  if ((state&(ControlMask|ShiftMask|Mod1Mask|Mod4Mask|Mod5Mask))==ShiftMask && ksym>=' ' && ksym < 0x7e) {
+    str = tt;
+    tt[0]=ksym;
+    tt[1]=0;
+    goto send_it;
+  }
+#endif
+
   return FALSE;
 }
