@@ -25,8 +25,7 @@ void all_wrap()
   gtk_text_buffer_apply_tag_by_name (buffer, "char_wrap", &mstart, &mend);
 }
 
-static char *A = "GB2312";
-static char *B = "big5";
+gboolean b_trad2sim = FALSE;
 
 
 static void selection_received(GtkClipboard *pclip, const gchar *text, gpointer data)
@@ -36,47 +35,15 @@ static void selection_received(GtkClipboard *pclip, const gchar *text, gpointer
     return;
   }
 
-  char *txt = strdup(text);
+  char *out;
+  if (b_trad2sim)
+    trad2sim(text, strlen(text), &out);
+  else
+    sim2trad(text, strlen(text), &out);
 
-  int len = strlen(text);
-  GError *err = NULL;
-  u_int rn = 0, wn = 0;
+  gtk_text_buffer_set_text (buffer, out, -1);
+  free(out);
 
-  char *gb;
-
-  do {
-    rn = wn = 0;
-    err = NULL;
-    gb = g_convert(txt, len, A , "UTF-8", &rn, &wn, &err);
-
-    if (err) {
-      dbg("utf8 -> %s  convert error %d %d\n", A, rn, wn);
-      txt[rn]=' ';
-    }
-  } while (err);
-
-  free(txt);
-
-  char *big5 = g_convert(gb, strlen(gb), B, A, &rn, &wn, &err);
-  g_free(gb);
-
-  if (err) {
-    dbg("%s -> %s  convert error %d %d\n", A, B, rn, wn);
-    return;
-  }
-
-  char *big5utf8 = g_convert(big5, strlen(big5), "UTF-8", B, &rn, &wn, &err);
-  g_free(big5);
-
-  if (err) {
-    dbg("%s -> %sutf8  convert error %d %d\n", B, B, rn, wn);
-    return;
-  }
-
-
-  gtk_text_buffer_set_text (buffer, big5utf8, -1);
-
-  g_free(big5utf8);
   all_wrap();
 }
 
@@ -105,10 +72,7 @@ int main(int argc, char **argv)
   gtk_init (&argc, &argv);
 
   if (strstr(argv[0],"trad2sim")) {
-    char *t = A;
-    A = B;
-    B = t;
-
+    b_trad2sim= TRUE;
     dbg("trad2sim\n");
   }
 
