@@ -3,6 +3,7 @@
 #include "gtab.h"
 #include "win-sym.h"
 #include "eggtrayicon.h"
+#include <signal.h>
 
 static GdkPixbuf *pixbuf, *pixbuf_ch;
 static PangoLayout* pango;
@@ -25,7 +26,7 @@ static void get_text_w_h(char *s, int *w, int *h)
 }
 
 #if USE_TSIN
-extern gboolean eng_ph;
+extern gboolean eng_ph, tsin_half_full;
 #endif
 
 static void draw_icon()
@@ -57,7 +58,7 @@ static void draw_icon()
   }
 
   if (current_CS) {
-    if (current_CS->b_half_full_char) {
+    if (current_CS->b_half_full_char || current_CS->in_method==6 && tsin_half_full && current_CS->im_state == GCIN_STATE_CHINESE) {
       static char full[] = "全";
       get_text_w_h(full,  &w, &h);
       gdk_draw_layout(tray_da_win, gc, dw - w, dh - h, pango);
@@ -69,7 +70,7 @@ static void draw_icon()
       gdk_draw_layout(tray_da_win, gc, 0, 0, pango);
     }
 #if USE_TSIN
-    if (current_CS->in_method==6 && current_CS->im_state == GCIN_STATE_CHINESE && !eng_ph) {
+    if ((current_CS->in_method==6||current_CS->in_method==12) && current_CS->im_state == GCIN_STATE_CHINESE && !eng_ph) {
       static char efull[] = "ABC";
       gdk_color_parse("blue", &color_fg);
       gdk_gc_set_rgb_fg_color(gc, &color_fg);
@@ -166,6 +167,31 @@ static void cb_tog_phospeak(GtkCheckMenuItem *checkmenuitem, gpointer *dat)
   phonetic_speak= gtk_check_menu_item_get_active(checkmenuitem);
 }
 
+
+void restart_gcin()
+{
+  static char execbin[]=GCIN_BIN_DIR"/gcin";
+
+  signal(SIGCHLD, SIG_IGN);
+
+  int pid = fork();
+  if (!pid) {
+    close_all_clients();
+    sleep(1);
+    execl(execbin, "gcin", NULL);
+  } else
+    exit(0);
+}
+
+
+static void cb_tog_gcb(GtkCheckMenuItem *checkmenuitem, gpointer *dat)
+{
+  gcb_enabled = gtk_check_menu_item_get_active(checkmenuitem);
+//  dbg("gcb_enabled %d\n", gcb_enabled);
+  gcb_main();
+}
+
+
 void kbm_toggle();
 
 struct {
@@ -174,11 +200,13 @@ struct {
   void (*cb)();
   int *check_dat;
 } mitems[] = {
-  {"設定", GTK_STOCK_PREFERENCES, exec_gcin_setup},
-  {"念出發音", NULL, cb_tog_phospeak, &phonetic_speak},
-  {"正->簡體", NULL, cb_trad2sim},
-  {"簡->正體", NULL, cb_sim2trad},
-  {"小鍵盤", NULL, kbm_toggle},
+  {N_("設定"), GTK_STOCK_PREFERENCES, exec_gcin_setup},
+  {N_("重新執行gcin"), NULL, restart_gcin},
+  {N_("念出發音"), NULL, cb_tog_phospeak, &phonetic_speak},
+  {N_("gcb(剪貼區暫存)"), NULL, cb_tog_gcb, &gcb_enabled},
+  {N_("正->簡體"), NULL, cb_trad2sim},
+  {N_("簡->正體"), NULL, cb_sim2trad},
+  {N_("小鍵盤"), NULL, kbm_toggle},
 };
 
 
@@ -194,16 +222,16 @@ static void create_menu()
     GtkWidget *item;
 
     if (mitems[i].stock_id) {
-      item = gtk_image_menu_item_new_with_label (mitems[i].name);
+      item = gtk_image_menu_item_new_with_label (_(mitems[i].name));
       gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), gtk_image_new_from_stock(mitems[i].stock_id, GTK_ICON_SIZE_MENU));
     }
     else
     if (mitems[i].check_dat) {
-      item = gtk_check_menu_item_new_with_label (mitems[i].name);
+      item = gtk_check_menu_item_new_with_label (_(mitems[i].name));
       gtk_check_menu_item_set_active(item, *mitems[i].check_dat);
     }
     else
-      item = gtk_menu_item_new_with_label (mitems[i].name);
+      item = gtk_menu_item_new_with_label (_(mitems[i].name));
 
     g_signal_connect (G_OBJECT (item), "activate",
                       G_CALLBACK (mitems[i].cb), NULL);
