@@ -7,7 +7,6 @@ Window xwin0;
 extern Display *dpy;
 static GtkWidget *top_bin;
 static GtkWidget *image_pin;
-int current_pho_simple_win;
 int current_gcin_inner_frame;
 
 static GtkWidget *hbox_edit;
@@ -44,14 +43,12 @@ static void recreate_win0()
 
 void change_win0_style()
 {
-  if (!top_bin || (current_pho_simple_win == pho_simple_win &&
-      current_gcin_inner_frame == gcin_inner_frame))
+  if (!top_bin || current_gcin_inner_frame == gcin_inner_frame)
     return;
 
   gtk_widget_destroy(top_bin);
   top_bin = NULL;
 
-  current_pho_simple_win = pho_simple_win;
   current_gcin_inner_frame = gcin_inner_frame;
   recreate_win0();
 }
@@ -127,6 +124,9 @@ void disp_char(int index, char *ch)
 {
   char tt[CH_SZ+1];
 
+  if (gcin_edit_display & GCIN_EDIT_DISPLAY_ON_THE_SPOT)
+    return;
+
 //  dbg("disp_char %d %c%c%c\n", index, ch[0], ch[1], ch[2]);
   create_char(index);
   GtkWidget *label = chars[index].label;
@@ -172,6 +172,9 @@ void draw_underline(int index)
 {
   create_char(index);
 
+  if (gcin_edit_display & GCIN_EDIT_DISPLAY_ON_THE_SPOT)
+    return;
+
   gtk_widget_show(chars[index].line);
 }
 
@@ -187,6 +190,9 @@ void set_cursor_tsin(int index)
   if (!label)
     return;
 
+  if (gcin_edit_display & GCIN_EDIT_DISPLAY_ON_THE_SPOT)
+    return;
+
   gtk_label_set_attributes(GTK_LABEL(label), attr_list);
 }
 
@@ -217,12 +223,43 @@ void clr_in_area_pho_tsin()
    disp_tsin_pho(i, " ");
 }
 
-
-void get_char_index_xy(int index, int *rx, int *ry)
+void get_widget_xy(GtkWidget *win, GtkWidget *widget, int *rx, int *ry)
 {
-  int wx, wy;
   int ofs=0;
+  gtk_widget_show_all(widget);
+  gdk_flush();
 
+  GtkRequisition sz;
+  gtk_widget_size_request(widget, &sz);
+  int wx, wy;
+
+  wx=wy=0;
+
+  gtk_widget_translate_coordinates(widget, win,
+         0, sz.height, &wx, &wy);
+
+  gtk_widget_translate_coordinates(widget, win,
+         0, sz.height, &wx, &wy);
+
+//  dbg("%d wx:%d\n", index,  wx);
+
+  int win_x, win_y;
+
+  gtk_window_get_position(GTK_WINDOW(win), &win_x, &win_y);
+  int win_xl, win_yl;
+  get_win_size(win, &win_xl, &win_yl);
+
+  if (wx > win_xl)
+    wx = win_xl;
+
+  *rx = win_x + wx;
+  *ry = win_y + wy + ofs;
+}
+
+void disp_tsin_select(int index)
+{
+  int x,y;
+  int ofs=0;
 #if 0
   GtkWidget *widget =chars[index].line;
 
@@ -233,45 +270,15 @@ void get_char_index_xy(int index, int *rx, int *ry)
 #endif
 #if 0
   GtkWidget *widget = chars[index].label;
-#endif
-#if 1
+#else
   GtkWidget *widget = chars[index].vbox;
 #endif
 
-  gtk_widget_show(widget);
-
-  GtkRequisition sz;
-  gtk_widget_size_request(widget, &sz);
-
-  gtk_widget_translate_coordinates(widget, gwin0,
-         0, sz.height, &wx, &wy);
-
-//  dbg("%d wx:%d\n", index,  wx);
-
-  int win_x, win_y;
-
-  gtk_window_get_position(GTK_WINDOW(gwin0), &win_x, &win_y);
-
-  *rx = win_x + wx;
-  *ry = win_y + wy + ofs;
-}
-
-void move_win_char_index(GtkWidget *win1, int index)
-{
-  int x,y;
-
-  get_char_index_xy(index, &x, &y);
-
-  int win1_xl, win1_yl;
-  get_win_size(win1, &win1_xl, &win1_yl);
-
-  if (x + win1_xl > dpy_xl)
-    x = dpy_xl - win1_xl;
-  if (y + win1_yl > dpy_yl)
-    y = win_y - win1_yl;
-
-//  dbg("move_win_char_index:%d %d\n", index, x);
-  gtk_window_move(GTK_WINDOW(win1), x, y);
+  if (gcin_edit_display & GCIN_EDIT_DISPLAY_ON_THE_SPOT) {
+    getRootXY(current_CS->client_win, current_CS->spot_location.x, current_CS->spot_location.y, &x, &y);
+  } else
+    get_widget_xy(gwin0, widget, &x, &y);
+  disp_selections(x, y);
 }
 
 #define MIN_X_SIZE 32
@@ -295,12 +302,15 @@ static void raw_move(int x, int y)
 void compact_win0_x()
 {
   int win_xl, win_yl;
-
+#if 0
   gtk_window_get_size(GTK_WINDOW(gwin0), &win_xl, &win_yl);
   if (max_yl < win_yl)
     max_yl = win_yl;
-
   gtk_window_resize(GTK_WINDOW(gwin0), MIN_X_SIZE, max_yl);
+#else
+  gtk_window_resize(GTK_WINDOW(gwin0), 16, 16);
+#endif
+
   raw_move(best_win_x, best_win_y);
 }
 
@@ -541,37 +551,14 @@ static void create_win0_gui()
   set_label_font_size(label_pho, gcin_font_size_tsin_pho_in);
   gtk_container_add (GTK_CONTAINER (button_pho), label_pho);
 
-#if 0
-  if (!pho_simple_win) {
-    GtkWidget *hbox_row2 = gtk_hbox_new (FALSE, 0);
-  //  gtk_container_set_border_width (GTK_CONTAINER (hbox_row2), 0);
-    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_row2, FALSE, FALSE, 0);
-    GtkWidget *button_tsin = gtk_button_new_with_label("詞音");
-    g_signal_connect_swapped (GTK_OBJECT (button_tsin), "button_press_event",
-          G_CALLBACK (inmd_switch_popup_handler), NULL);
-    gtk_box_pack_start (GTK_BOX (hbox_row2), button_tsin, FALSE, FALSE, 0);
-
-
-    button_eng_ph = gtk_button_new_with_label("注");
-    gtk_box_pack_start (GTK_BOX (hbox_row2), button_eng_ph, FALSE, FALSE, 0);
-    g_signal_connect (G_OBJECT (button_eng_ph), "clicked",
-        G_CALLBACK (cb_clicked_eng_ph), (gpointer) NULL);
-
-    image_pin = gtk_image_new_from_file(file_pin_float);
-    GtkWidget *event_box_pin = gtk_event_box_new();
-    gtk_container_add (GTK_CONTAINER (event_box_pin), image_pin);
-    gtk_box_pack_start (GTK_BOX (hbox_row2), event_box_pin, FALSE, FALSE, 0);
-    g_signal_connect (G_OBJECT (event_box_pin), "button_press_event",
-        G_CALLBACK (cb_clicked_fixed_pos), (gpointer) NULL);
-  }
-#endif
-
   clr_in_area_pho_tsin();
 
   gtk_widget_show_all (gwin0);
   gdk_flush();
   gtk_widget_hide (gwin0);
 
+//  compact_win0();
+
   create_win1();
   create_win1_gui();
 
@@ -587,6 +574,8 @@ void destroy_win0()
     return;
 
   gtk_widget_destroy(gwin0);
+  gwin0 = NULL;
+  top_bin = NULL;
 }
 
 void get_win0_geom()
@@ -600,16 +589,14 @@ gboolean tsin_has_input(), force_show;
 
 void show_win0()
 {
-//  dbg("show_win0\n");
   create_win0();
   create_win0_gui();
-//  set_currenet_IC_pin_image_pin();
 
   if (gcin_pop_up_win && !tsin_has_input() && !force_show)
     return;
 
-  gtk_widget_show(gwin0);
-//  dbg("show_win0 b\n");
+  if (!GTK_WIDGET_VISIBLE(gwin0))
+    gtk_widget_show(gwin0);
 
   show_win_sym();
   if (current_CS->b_raise_window) {
@@ -622,6 +609,8 @@ void hide_selections_win();
 void hide_win0()
 {
 //  dbg("hide_win0\n");
+  if (!gwin0)
+    return;
   gtk_widget_hide(gwin0);
   hide_selections_win();
   hide_win_sym();
@@ -690,3 +679,15 @@ void change_tsin_color()
 
   drawcursor();
 }
+
+
+void recreate_tsin_select_win();
+#if 0
+void recreate_tsin_win()
+{
+  destroy_win0();
+  create_win0();
+  create_win0_gui();
+  recreate_tsin_select_win();
+}
+#endif
