@@ -33,7 +33,6 @@ static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpoi
       tsin_sele_by_idx(v);
       break;
   }
-
 }
 
 void create_win1_gui()
@@ -108,6 +107,7 @@ void clear_sele()
   gtk_widget_hide(arrow_up);
   gtk_widget_hide(arrow_down);
   gtk_window_resize(GTK_WINDOW(gwin1), 10, 20);
+  hide_selections_win();
 }
 
 char *htmlspecialchars(char *s, char out[]);
@@ -141,18 +141,21 @@ void set_sele_text(int i, char *text, int len)
 }
 
 
-void move_win_char_index(GtkWidget *win1, int index);
 
-void disp_selections(int idx)
+void disp_selections(int x, int y)
 {
-  if (idx < 0)
-    p_err("err");
+  if (!GTK_WIDGET_VISIBLE(gwin1))
+    gtk_widget_show(gwin1);
 
-  move_win_char_index(gwin1, idx);
-  gtk_widget_show(gwin1);
-#if 1 // strange bug in gtk
-  move_win_char_index(gwin1, idx);
-#endif
+  int win1_xl, win1_yl;
+  get_win_size(gwin1, &win1_xl, &win1_yl);
+
+  if (x + win1_xl > dpy_xl)
+    x = dpy_xl - win1_xl;
+  if (y + win1_yl > dpy_yl)
+    y = win_y - win1_yl;
+
+  gtk_window_move(GTK_WINDOW(gwin1), x, y);
 }
 
 void raise_tsin_selection_win()
@@ -185,6 +188,8 @@ void destroy_win1()
   if (!gwin1)
     return;
   gtk_widget_destroy(gwin1);
+  frame=NULL;
+  gwin1 = NULL;
 }
 
 void change_win1_font()
@@ -197,8 +202,10 @@ void change_win1_font()
   for(i=0; i < SELEN; i++) {
     set_label_font_size(labels_sele[i], gcin_font_size_tsin_presel);
     set_label_font_size(labels_seleR[i], gcin_font_size_tsin_presel);
-    gtk_widget_modify_fg(labels_sele[i], GTK_STATE_NORMAL, gcin_win_color_use?&fg:NULL);
-    gtk_widget_modify_fg(labels_seleR[i], GTK_STATE_NORMAL, gcin_win_color_use?&fg:NULL);
+    if (labels_sele[i])
+      gtk_widget_modify_fg(labels_sele[i], GTK_STATE_NORMAL, gcin_win_color_use?&fg:NULL);
+    if (labels_seleR[i])
+      gtk_widget_modify_fg(labels_seleR[i], GTK_STATE_NORMAL, gcin_win_color_use?&fg:NULL);
     change_win_bg(eve_sele[i]);
     change_win_bg(eve_seleR[i]);
   }
